#
# Copyright (C) 2017 Nethesis S.r.l.
# http://www.nethesis.it - nethserver@nethesis.it
#
# This script is part of NethServer.
#
# NethServer is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License,
# or any later version.
#
# NethServer is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with NethServer.  If not, see COPYING.
#

package NethServer::NethBackup;

use strict;
use URI::Escape;
use JSON;
use NethServer::BackupConfig;
use esmith::ConfigDB;

sub prepare_curl_args
{
    my @curlArgs = (
        '--silent', '--show-error',
        '--user-agent', 'NethBackup',
        '--request', 'POST',
        '--location-trusted',
        '--url', 'https://backupd.nethesis.it/backup',
        @_,
    );

    my $cdb = esmith::ConfigDB->open_ro();
    if($cdb) {
        my $proxy_host = $cdb->get_prop('proxy', 'host') || '';
        my $proxy_port = $cdb->get_prop('proxy', 'port') || '3128';
        my $proxy_user = $cdb->get_prop('proxy', 'user') || '';
        my $proxy_pass = $cdb->get_prop('proxy', 'password') || '';

        if ($proxy_host ne '') {
            push @curlArgs, '--proxy', "$proxy_host:$proxy_port";
            if ($proxy_user ne '' && $proxy_pass ne '') {
                push @curlArgs, '--proxy-user', "$proxy_user:$proxy_pass";
            }
        }
    }

    return @curlArgs;
}

sub curl_exec
{
    my @curlArgs = prepare_curl_args(@_);
    $ENV{'NETHBACKUP_DEBUG'} && warn("[DEBUG] " . join(" ", 'curl', map { "'$_'" } @curlArgs) . "\n");

    open my $fh, '-|', '/usr/bin/curl', @curlArgs;
    my $payload = do {
        local $/;
        $/ = undef;
        <$fh>
    };
    my $is_success = close($fh);

    $ENV{'NETHBACKUP_DEBUG'} && warn("[DEBUG] success:$is_success payload:" . ($payload ? (($payload =~ m/[^\x0a\x0d\x20-\x7f]/m) ? 'bInary dAta' : $payload) : 'emPty') . "\n");
    return $payload, $is_success;
}

sub backup_list
{
    my ($key, $secret) = @_;

    my ($rescon, $is_success) = curl_exec(
        '--user', "$key:$secret",
        '--header', 'NethBackup-Action: backup-list',
        '--header', 'NethBackup-Parameters: LK=' . uri_escape($key),
    );
    chomp($rescon);

    return map { [split(/;/, $_, 2)] } split(/\|/, $rescon);
}


sub get_backup
{
    my ($key, $secret, $bakfile, $fpath) = @_;
    my $umask_orig = umask 037;

    my ($rescon, $is_success) = curl_exec(
        '--fail', # capture is_success from curl exit code
        '--user', "$key:$secret",
        '--header', 'NethBackup-Action: backup-get',
        '--header', "NethBackup-Parameters: LK=$key&bakfile=$bakfile",
        '--output', $fpath,
    );

    umask $umask_orig;

    if (-z $fpath || !$is_success) {
        return 0;
    }

    return 1;
}

sub send_bck
{
    my ($key, $secret, $filename, $usefname, $desc, $retry) = @_;

    $retry = 0 if (!$retry);

    $filename =~ /^.*\/(.*)$/; # cutting path
    my $fname = $1;

    my ($rescon, $is_success) = curl_exec(
        '--user', "$key:$secret",
        '--header', 'NethBackup-Action: backup-send',
        '--header', 'NethBackup-Parameters: LK=' . uri_escape($key) . ($usefname ? "&fname=". uri_escape($fname) : '') . ($desc ? "&desc=". uri_escape($desc) : ''),
        '--data-binary', '@' . $filename,
    );

    chomp($rescon);
    if ($rescon =~ /success/) {
        return 1;
    } elsif ($rescon =~ /error.*code\s*:\s*'(.*)'/) {
        print "Error: $1". "\n";
        if ($retry < 2) {
            print "Trying to resend..\n";
            return send_bck($key, $secret, $filename, $usefname, $desc, ++$retry);
        }
    }
    return 0;
}

1;
